package cs4621.lecture02;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLProfile;
import javax.swing.JFrame;
import javax.swing.Timer;


public class Demo11 extends JFrame implements GLEventListener, ActionListener 
{
	public static int CORNERS = 256;
	
	private class Ball {
		public double x, y;
		public double vx, vy;
		public double radius;
		public double r, g, b;	
		
		public Ball() {
			x = 0;
			y = 0;
			vx = 0.02;
			vy = 0.03;
			radius = 0.1;
			r = 1.0;
			g = 0.5;
			b = 0.5;
		}
		
		public void draw(GL2 gl)
		{
			gl.glColor3d(r, g, b);
			gl.glBegin(GL2.GL_TRIANGLE_FAN);
			{
				gl.glVertex2d(x, y);
				for(int i=0;i<CORNERS;i++)
				{
					double theta = 2*Math.PI*i/CORNERS;
					double xx = x + radius * Math.cos(theta);
					double yy = y + radius * Math.sin(theta);
					gl.glVertex2d(xx,yy);
				}
				gl.glVertex2d(x + radius, y);
			}
			gl.glEnd();
		}
		
		public void update()
		{
			x += vx;
			y += vy;

			if (x + radius > 1)
			{
				x = 1 - radius;
				vx = -Math.abs(vx);
			}
			if (x - radius < -1)
			{
				x = -1 + radius;
				vx = Math.abs(vx);
			}
			if (y + radius > 1)
			{
				y = 1 - radius;
				vy = -Math.abs(vy);
			}
			if (y - radius < -1)
			{
				y = -1 + radius;
				vy = Math.abs(vy);
			}
		}
	}
	
	private Timer timer;
	private GLCanvas canvas;
	private Ball ball;
	
	public Demo11() {		
		super("CS 4621 Demo 11: How to animate with Timer.");
		
		ball = new Ball();
		
		GLProfile profile = GLProfile.getDefault();
		GLCapabilities capabilities = new GLCapabilities(profile);
		canvas = new GLCanvas(capabilities);
		canvas.addGLEventListener(this);
				
		getContentPane().add(canvas, BorderLayout.CENTER);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);		
	}
	
	public void run()
	{
		setSize(512, 512);
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	public static void main(String[] args) {		
		new Demo11().run();		
	}

	@Override
	public void init(GLAutoDrawable drawable) {		
		timer = new Timer(33, this);		
		timer.start();		
	}	

	@Override
	public void display(GLAutoDrawable drawable) {
		final GL2 gl = drawable.getGL().getGL2();
		
		gl.glClearColor(0, 0, 0, 0);
		gl.glClear(GL2.GL_COLOR_BUFFER_BIT);
		
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
		ball.draw(gl);
	}

	@Override
	public void reshape(GLAutoDrawable drawble, 
			int x, int y, int width, int height) {
		// NOP
	}

	@Override
	public void dispose(GLAutoDrawable drawable) {
		// NOP		
	}

	@Override
	public void actionPerformed(ActionEvent ev) {
		if (ev.getSource() == timer)
		{			
			ball.update();
			canvas.repaint();
		}
	}
}
