package cs4621.lecture02;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLProfile;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.Timer;

import com.jogamp.opengl.util.FPSAnimator;


public class Demo14 extends JFrame implements GLEventListener, ActionListener
{		
	private Timer timer;
	private GLCanvas canvas;
	private JButton toggleProjectionButton;
	private boolean useOrthographic;
	
	private double rotateX = 0;
	private double rotateY = 0;
	private double rotateZ = 0;
	
	public Demo14() {		
		super("CS 4621 Demo 14: Orthographic and Perspective Projections");		
		
		GLProfile profile = GLProfile.getDefault();
		GLCapabilities capabilities = new GLCapabilities(profile);
		canvas = new GLCanvas(capabilities);
		canvas.addGLEventListener(this);
						
		getContentPane().add(canvas, BorderLayout.CENTER);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);		
		
		toggleProjectionButton = new JButton("Toggle Projection: Orthographic");
		toggleProjectionButton.addActionListener(this);
		getContentPane().add(toggleProjectionButton, BorderLayout.SOUTH);
		useOrthographic = true;
		
		timer = new Timer(30, this);
	}
	
	public void run()
	{
		setSize(512, 512);
		setLocationRelativeTo(null);		
		setVisible(true);
	}
	
	public static void main(String[] args) {		
		new Demo14().run();		
	}

	@Override
	public void init(GLAutoDrawable drawable) {		
		timer.start();
	}	

	@Override
	public void display(GLAutoDrawable drawable) {
		final GL2 gl = drawable.getGL().getGL2();
		GLU glu = new GLU();
		
		gl.glEnable(GL2.GL_DEPTH_TEST);
		gl.glClearColor(0, 0, 0, 0);
		gl.glClearDepth(1.0);			
		gl.glClear(GL2.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT);
		
		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		if (useOrthographic)
		{
			gl.glOrtho(-4.5, 4.5, -4.5, 4.5, 0.1, 100);
		}
		else
		{
			glu.gluPerspective(45, 1, 0.1, 100);
		}
		
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
				
		glu.gluLookAt(0.0, 0.0, 10.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
		
		gl.glRotated(rotateX, 1, 0, 0);
		gl.glRotated(rotateY, 0, 1, 0);
		gl.glRotated(rotateZ, 0, 0, 1);
		
		drawCube(gl);
	}
	
	protected void drawCube(GL2 gl){		
		gl.glBegin(GL2.GL_QUADS);					
			gl.glColor3f(0.0f,1.0f,0.0f);			
			gl.glVertex3f( 1.0f, 1.0f,-1.0f);			
			gl.glVertex3f(-1.0f, 1.0f,-1.0f);			
			gl.glVertex3f(-1.0f, 1.0f, 1.0f);			
			gl.glVertex3f( 1.0f, 1.0f, 1.0f);		
			
			gl.glColor3f(1.0f,0.5f,0.0f);			
			gl.glVertex3f( 1.0f,-1.0f, 1.0f);			
			gl.glVertex3f(-1.0f,-1.0f, 1.0f);			
			gl.glVertex3f(-1.0f,-1.0f,-1.0f);			
			gl.glVertex3f( 1.0f,-1.0f,-1.0f);
			
			gl.glColor3f(1.0f,0.0f,0.0f);			
			gl.glVertex3f( 1.0f, 1.0f, 1.0f);			
			gl.glVertex3f(-1.0f, 1.0f, 1.0f);			
			gl.glVertex3f(-1.0f,-1.0f, 1.0f);			
			gl.glVertex3f( 1.0f,-1.0f, 1.0f);
			
			gl.glColor3f(1.0f,1.0f,0.0f);			
			gl.glVertex3f( 1.0f,-1.0f,-1.0f);			
			gl.glVertex3f(-1.0f,-1.0f,-1.0f);			
			gl.glVertex3f(-1.0f, 1.0f,-1.0f);			
			gl.glVertex3f( 1.0f, 1.0f,-1.0f);
			
			gl.glColor3f(0.0f,0.0f,1.0f);			
			gl.glVertex3f(-1.0f, 1.0f, 1.0f);			
			gl.glVertex3f(-1.0f, 1.0f,-1.0f);			
			gl.glVertex3f(-1.0f,-1.0f,-1.0f);			
			gl.glVertex3f(-1.0f,-1.0f, 1.0f);
			
			gl.glColor3f(1.0f,0.0f,1.0f);			
			gl.glVertex3f( 1.0f, 1.0f,-1.0f);			
			gl.glVertex3f( 1.0f, 1.0f, 1.0f);			
			gl.glVertex3f( 1.0f,-1.0f, 1.0f);			
			gl.glVertex3f( 1.0f,-1.0f,-1.0f);			
		gl.glEnd();	
	}

	@Override
	public void reshape(GLAutoDrawable drawble, 
			int x, int y, int width, int height) {
		// NOP
	}

	@Override
	public void dispose(GLAutoDrawable drawable) {
		// NOP		
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == toggleProjectionButton)
		{
			if (useOrthographic)
			{
				toggleProjectionButton.setText("Toggle Projection: Perspective");
				useOrthographic = false;
				canvas.repaint();
			}
			else
			{
				toggleProjectionButton.setText("Toggle Projection: Orthographic");
				useOrthographic = true;
				canvas.repaint();
			}
		}
		else if (e.getSource() == timer)
		{
			rotateX += 0.1;
			rotateY += 0.2;
			rotateZ += 0.3;
			canvas.repaint();
		}
	}	
}
